'use strict';

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _bodyParser = require('body-parser');

var _bodyParser2 = _interopRequireDefault(_bodyParser);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _args = require('../util/args');

var _args2 = _interopRequireDefault(_args);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const DEFAULT_PORT = 8081;
const DEFAULT_STORE_PATH = '/tmp/colosseum_data/default';

const create = () => {
  let started = false;
  const handlers = {};
  let host;
  let key;
  let storePath;
  let initialisedStore = false;

  const handle = (message, handler) => {
    handlers[message] = handler;
  };

  const message = (service, messageName, payload = {}) => new Promise((resolve, reject) => {
    if (started) {
      if (!host) {
        reject('Missing host.');
      } else if (!key) {
        reject('Missing key.');
      } else {
        const requestUrl = `${host}/${service}/${messageName}/${key}`;
        _request2.default.post(requestUrl, { json: payload }, (error, response, body) => {
          if (error) {
            reject(error);
          } else if (body) {
            if (body.success) {
              resolve(body.response);
            } else {
              reject(body.error);
            }
          } else {
            reject('Invalid response');
          }
        });
      }
    } else {
      reject('Not started');
    }
  });

  const initialiseStore = async () => {
    if (!initialisedStore) {
      _fsExtra2.default.ensureDirSync(storePath);
      initialisedStore = true;
    }
  };

  const write = async (name, data) => {
    await initialiseStore();
    const writePath = _path2.default.join(storePath, name);
    _fsExtra2.default.writeFileSync(writePath, data, 'utf8');
  };

  const start = () => {
    if (!started) {
      started = true;

      const args = (0, _args2.default)({
        host: null,
        port: DEFAULT_PORT,
        key: null,
        store: DEFAULT_STORE_PATH
      });

      const port = Number(args.port);
      host = args.host;
      key = args.key;
      storePath = args.store;

      const app = (0, _express2.default)();
      app.use(_bodyParser2.default.json());

      app.post('/:message/:key', async (req, res) => {
        try {
          if (key && req.params.key === key) {
            const payload = req.body || {};

            const handler = handlers[req.params.message];
            if (handler) {
              const response = await Promise.resolve(handler(payload));
              res.json({
                success: true,
                response
              });
            } else {
              res.status(400).json({
                success: false,
                error: 'No handler for message'
              });
            }
          } else {
            res.status(403).json({
              success: false,
              error: '🖕'
            });
          }
        } catch (error) {
          console.error(error.stack || error);
          res.status(500).json({
            success: false,
            error
          });
        }
      });

      app.listen(port, () => {
        message('colosseum', 'running', { key }).catch(() => null);
      });
    }
  };

  return {
    message,
    write,
    handle,
    start
  };
};

module.exports = create;