'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
const MetaService = (serviceManager, onServiceRunning) => {
  const register = async payload => {
    if (!payload.name) {
      return Promise.reject('Missing service name.');
    }
    if (!payload.url) {
      return Promise.reject('Missing service url.');
    }
    await serviceManager.register(payload.name, payload.url);
    return Promise.resolve('👌');
  };

  const status = () => serviceManager.getStatus();

  const registerService = payload => {
    if (!payload.name) {
      return Promise.reject('Missing service name.');
    }
    if (!payload.url) {
      return Promise.reject('Missing service url.');
    }
    return serviceManager.register(payload.name, payload.url);
  };

  const removeService = payload => {
    if (!payload.name) {
      return Promise.reject('Missing service name.');
    }
    return serviceManager.remove(payload.name);
  };

  const startService = payload => {
    if (!payload.name) {
      return Promise.reject('Missing service name.');
    }
    return serviceManager.start(payload.name);
  };

  const stopService = payload => {
    if (!payload.name) {
      return Promise.reject('Missing service name.');
    }
    return serviceManager.stop(payload.name);
  };

  const restartService = payload => {
    if (!payload.name) {
      return Promise.reject('Missing service name.');
    }
    return serviceManager.restart(payload.name);
  };

  const updateService = payload => {
    if (!payload.name) {
      return Promise.reject('Missing service name.');
    }
    return serviceManager.update(payload.name);
  };

  const getName = () => 'colosseum';

  const getState = () => 'running';

  const validateKey = () => false;

  const sendMessage = (message, payload = {}) => {
    if (message === 'register') {
      return register(payload);
    }
    if (message === 'status') {
      return status();
    }
    if (message === 'register') {
      return registerService(payload);
    }
    if (message === 'remove') {
      return removeService(payload);
    }
    if (message === 'start') {
      return startService(payload);
    }
    if (message === 'stop') {
      return stopService(payload);
    }
    if (message === 'restart') {
      return restartService(payload);
    }
    if (message === 'update') {
      return updateService(payload);
    }
    if (message === 'running') {
      return onServiceRunning(payload.key);
    }
    return Promise.reject('No handler for message');
  };

  return {
    getName,
    getState,
    validateKey,
    start: async () => null,
    stop: async () => null,
    restart: async () => null,
    update: async () => null,
    remove: async () => null,
    sendMessage
  };
};

exports.default = MetaService;