'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.run = exports.start = undefined;

var _child_process = require('child_process');

const start = (command, directory, onOutput, onError, onStopped) => {
  const commandParts = command.split(' ');
  let internalOnStopped;

  const process = (0, _child_process.spawn)(commandParts[0], commandParts.slice(1), {
    cwd: directory
  });

  process.stdout.on('data', data => {
    if (onOutput) {
      onOutput(data.toString());
    }
  });

  process.stderr.on('data', data => {
    if (onError) {
      onError(data.toString());
    }
  });

  process.on('close', code => {
    if (onStopped) {
      onStopped(code);
    }
    if (internalOnStopped) {
      internalOnStopped();
    }
  });

  const stop = () => new Promise(resolve => {
    internalOnStopped = resolve;
    process.kill();
  });

  return {
    stop
  };
};

const run = (command, directory, onOutput, onError) => new Promise(resolve => {
  start(command, directory, onOutput, onError, resolve);
});

exports.start = start;
exports.run = run;